************************************************************
* Name:        CALNDAR1.PRG
* Desc:        A popup calendar program.  User can change
*              the month or year by pressing the PgUp, PgDn,
*              + , -
*              RIGHT ARROW KEY ->  will advance calendar through days
*              CTRL-END will select a day and return to calling program.
*
* Author:      Todd W. Lindley
* Created:     3/12/91
*
* Modified by: Steven L. Baker, Data Design Services, 11/28/91
* BORBBS ID:   SPSLB (comments welcome)
* 
* MODIFICATIONS MADE ON 11/28/1991:
*              Modified for use with one memory variable (no dBASE file).
*
* Usage:       DO CALNDAR1 WITH Mdate
*
*              Ex: DO CALNDAR1 WITH Mdate
* Results:     Displays a popup calendar at screen coordinates 2,2 in large
*              format with the calendar displaying the month and year of
*              Mdate. Program will return whatever date was indicated
*              when the user pressed CTRL-END.
************************************************************

PROCEDURE CALNDAR1
  PARAMETERS TDATE              && Date, row, and column on screen

  SET TALK OFF
  SET STATUS OFF
  SAVE SCREEN TO Mscreen
  SET COLOR OF NORMAL TO W+/R    && Calendar will be white on red

  PUBLIC New_date
  STORE 0 TO MONTH,YEAR,DAY,FIRSTDAY,LASTDAY,SKIP_COLS,LAST_COL,COL,ROW,S_COL
  STORE 0 TO CURR,NEXT,X,MONTHN,LAST_ROW,INFO_ROW,S_ROW,E_ROW,S1_COL
  STORE 1 TO YY

  START_ROW = 4        && Starting row of calendar
  START_COL = 30       && Strating column of calendar

  DO init
  DO grid
  DO calc
  DO refresh
  S_ROW = START_ROW+ 3

* ---- MAIN LOOP - READS KEYBOARD ----------------------------
@ S_ROW,S_COL SAY " 1"  COLOR GR+/R
YY = 1
DO WHILE .T.
  KEY=INKEY()
  DO CASE

    *  ----- CTRL-END -----------------
    CASE key=23
      EXIT

    *  ----- right arrow---------------
    CASE key=4
          * ---  Rewrite old day with regular colors
      NEW_DATE = CTOD(STR(MONTH,2,0)+"/"+STR(YY,2,0)+"/"+STR(YEAR,4,0))
      @ S_ROW,S_COL SAY STR(YY,2,0)
      S_COL = S_COL+ SKIP_COLS
      YY = YY+1
      IF S_COL > LAST_COL
        S_COL = START_COL+3
        S_ROW = S_ROW+2
        IF S_ROW > E_ROW
          S_ROW = START_ROW + 3
          S_COL = S1_COL
          YY = 1
        ENDIF
      ENDIF
      IF YY > LASTDAY
        S_ROW = START_ROW+3
        S_COL = S1_COL
        YY = 1
      ENDIF
           *---  Write indicated day with bold colors
      NEW_DATE = CTOD(STR(MONTH,2,0)+"/"+STR(YY,2,0)+"/"+STR(YEAR,4,0))
      @ S_ROW,S_COL SAY STR(YY,2,0) COLOR GR+/R

    *  ----- page up next MONTH----------
    CASE key=18
      MONTH=MONTH+1
      IF MONTH>12
        MONTH=1
        YEAR=YEAR+1
      ENDIF
      DO calc
      DO refresh

    *  ----- page down previous MONTH---------------
    CASE key=3
      MONTH=MONTH-1
      IF MONTH<1
        MONTH=12
        YEAR=YEAR-1
      ENDIF
      DO calc
      DO refresh

    *  ----- + key next year ---------------
    CASE key=43
      YEAR=YEAR+1
      IF YEAR<0
        YEAR=1
      ENDIF
      DO calc
      DO refresh

    *  ----- - key previous year ---------------
    CASE key=45
      YEAR=YEAR-1
      DO calc
      DO refresh
  ENDCASE
ENDDO
  Tdate = New_date

  SET COLOR OF NORMAL TO W+/B      && Reset normal color
  RESTORE SCREEN FROM Mscreen
  RELEASE Mscreen , New_date
RETURN
*-- EOM: Calndar1


* --- procedure to refresh screen -------------
PROCEDURE refresh
  yy = 1
  x=1
  row=start_row+3
  @info_row,start_col+1 CLEAR TO info_row,start_col+skip_cols*6
  @info_row,start_col SAY monthn
  @info_row,last_col-3 SAY STR(year,4,0)
  DO clear_grid
  S_ROW = ROW
  S1_COL = COL

  DO WHILE x <= lastday
    NEW_DATE = CTOD(STR(MONTH,2,0)+"/"+STR(x,2,0)+"/"+STR(YEAR,4,0))
    @ ROW,COL SAY STR(x,2,0)
    COL=COL+SKIP_COLS
    IF COL > LAST_COL
      COL=START_COL+3
      ROW=ROW+2
    ENDIF
    X=X+1
  ENDDO

  E_ROW  = ROW
  NEW_DATE = CTOD(STR(MONTH,2,0)+"/"+STR(1,2,0)+"/"+STR(YEAR,4,0))
  @ S_ROW,S_COL SAY " 1" COLOR GR+/R
RETURN
*-- EOP: Refresh


* --- initial set up procedure  -------------
PROCEDURE init
  month=month(tdate)
  year=YEAR(tdate)
  day=DAY(tdate)
RETURN
*- EOP: Init


* --- procedure to calculate days of the month ---------
PROCEDURE calc
 ** GET MONTH NAME
  monthn=CMONTH(CTOD(STR(month,2,0)+"/01/91"))

 ** CALCULATE 1ST OF MONTH AND BEGINNING OF NEXT MONTH
  beg=CTOD(STR(month,2,0)+"/"+"01/"+STR(year,4,0))
  next=CTOD(STR(month+1,2,0)+"/"+"01/"+STR(year,4,0))

 ** CALCULATE # OF DAYS IN MONTH
  firstday=DOW(beg)
  lastday=next-beg

 ** STARTING DISPLAY COLUMN OF 1ST DAY OF MONTH
  col=skip_cols*(firstday-1)+start_col+3
  s_col = col
RETURN
*-- EOP: Calc


* --- procedure to rebuild grid -------------
PROCEDURE grid
** EACH @ SAY WAS HARD CODED FOR SPEED IN DRAWING THE GRID.
** A DO WHILE COULD HAVE BEEN USED BUT WOULD HAVE BEEN TOO SLOW.
  skip_cols=5
  last_col=start_col+skip_cols*6+5
  last_row=start_row+14
  info_row=start_row-1

  @start_row-1,start_col-1 CLEAR TO last_row+3,last_col+1
  @start_row+1,start_col+2 SAY "Sun  Mon  Tue  Wed  Thu  Fri  Sat"
  @start_row,start_col TO last_row,last_col
  @start_row+2,start_col TO start_row+2,last_col
  @start_row,start_col+skip_cols TO last_row,start_col+skip_cols*6
  @start_row,start_col+skip_cols*2 TO last_row,start_col+skip_cols*5
  @start_row,start_col+skip_cols*3 TO last_row,start_col+skip_cols*4

  @start_row,start_col+skip_cols SAY CHR(194)
  @start_row,start_col+skip_cols*2 SAY CHR(194)
  @start_row,start_col+skip_cols*3 SAY CHR(194)
  @start_row,start_col+skip_cols*4 SAY CHR(194)
  @start_row,start_col+skip_cols*5 SAY CHR(194)
  @start_row,start_col+skip_cols*6 SAY CHR(194)

  @start_row+2,start_col SAY CHR(195)
  @start_row+2,start_col+skip_cols SAY CHR(197)
  @start_row+2,start_col+skip_cols*2 SAY CHR(197)
  @start_row+2,start_col+skip_cols*3 SAY CHR(197)
  @start_row+2,start_col+skip_cols*4 SAY CHR(197)
  @start_row+2,start_col+skip_cols*5 SAY CHR(197)
  @start_row+2,start_col+skip_cols*6 SAY CHR(197)
  @start_row+2,last_col SAY CHR(180)

  @start_row+4,start_col TO last_row-2,last_col
  @start_row+6,start_col TO last_row-4,last_col
  @start_row+8,start_col TO last_row-6,last_col

  @start_row+4,start_col SAY CHR(195)
  @start_row+4,start_col+skip_cols SAY CHR(197)
  @start_row+4,start_col+skip_cols*2 SAY CHR(197)
  @start_row+4,start_col+skip_cols*3 SAY CHR(197)
  @start_row+4,start_col+skip_cols*4 SAY CHR(197)
  @start_row+4,start_col+skip_cols*5 SAY CHR(197)
  @start_row+4,start_col+skip_cols*6 SAY CHR(197)
  @start_row+4,last_col SAY CHR(180)

  @start_row+6,start_col SAY CHR(195)
  @start_row+6,start_col+skip_cols SAY CHR(197)
  @start_row+6,start_col+skip_cols*2 SAY CHR(197)
  @start_row+6,start_col+skip_cols*3 SAY CHR(197)
  @start_row+6,start_col+skip_cols*4 SAY CHR(197)
  @start_row+6,start_col+skip_cols*5 SAY CHR(197)
  @start_row+6,start_col+skip_cols*6 SAY CHR(197)
  @start_row+6,last_col SAY CHR(180)

  @start_row+8,start_col SAY CHR(195)
  @start_row+8,start_col+skip_cols SAY CHR(197)
  @start_row+8,start_col+skip_cols*2 SAY CHR(197)
  @start_row+8,start_col+skip_cols*3 SAY CHR(197)
  @start_row+8,start_col+skip_cols*4 SAY CHR(197)
  @start_row+8,start_col+skip_cols*5 SAY CHR(197)
  @start_row+8,start_col+skip_cols*6 SAY CHR(197)
  @start_row+8,last_col SAY CHR(180)

  @start_row+10,start_col SAY CHR(195)
  @start_row+10,start_col+skip_cols SAY CHR(197)
  @start_row+10,start_col+skip_cols*2 SAY CHR(197)
  @start_row+10,start_col+skip_cols*3 SAY CHR(197)
  @start_row+10,start_col+skip_cols*4 SAY CHR(197)
  @start_row+10,start_col+skip_cols*5 SAY CHR(197)
  @start_row+10,start_col+skip_cols*6 SAY CHR(197)
  @start_row+10,last_col SAY CHR(180)

  @start_row+12,start_col SAY CHR(195)
  @start_row+12,start_col+skip_cols SAY CHR(197)
  @start_row+12,start_col+skip_cols*2 SAY CHR(197)
  @start_row+12,start_col+skip_cols*3 SAY CHR(197)
  @start_row+12,start_col+skip_cols*4 SAY CHR(197)
  @start_row+12,start_col+skip_cols*5 SAY CHR(197)
  @start_row+12,start_col+skip_cols*6 SAY CHR(197)
  @start_row+12,last_col SAY CHR(180)

  @last_row,start_col SAY CHR(192)
  @last_row,start_col+skip_cols SAY CHR(193)
  @last_row,start_col+skip_cols*2 SAY CHR(193)
  @last_row,start_col+skip_cols*3 SAY CHR(193)
  @last_row,start_col+skip_cols*4 SAY CHR(193)
  @last_row,start_col+skip_cols*5 SAY CHR(193)
  @last_row,start_col+skip_cols*6 SAY CHR(193)
  @last_row,last_col SAY CHR(217)

*-- Shadow for calendar
  @start_row,last_col+2 FILL TO last_row+4,last_col+3   COLOR N/N
  @last_row+4,start_col+1 FILL TO last_row+4,last_col+3   COLOR N/N

*-- User help information at bottom of calendar
  @last_row+1,start_col+1 SAY "Month:"
  @last_row+1,start_col+7 SAY "PgUp/PgDn" COLOR GR+/R
  @last_row+1,start_col+19 SAY "Day:"
  @last_row+1,start_col+23 SAY "->" COLOR GR+/R
  @last_row+1,last_col-8 SAY "Year:"
  @last_row+1,last_col-3 SAY "+/-" COLOR GR+/R
  @last_row+2,start_col+1 SAY "Select day (and close):"
  @last_row+2,start_col+25 SAY "CTRL-END" COLOR GR+/R
RETURN
*-- EOP: Grid


* --- procedure to clear grid -------------
PROCEDURE clear_grid
  @row,start_col+skip_cols-2 SAY "  "
  @row,start_col+skip_cols*2-2 SAY "  "
  @row,start_col+skip_cols*3-2 SAY "  "
  @row,start_col+skip_cols*4-2 SAY "  "
  @row,start_col+skip_cols*5-2 SAY "  "
  @row,start_col+skip_cols*6-2 SAY "  "
  @row,start_col+skip_cols*7-2 SAY "  "

  @last_row-1,start_col+skip_cols-2 SAY "  "
  @last_row-1,start_col+skip_cols*2-2 SAY "  "
  @last_row-1,start_col+skip_cols*3-2 SAY "  "
  @last_row-1,start_col+skip_cols*4-2 SAY "  "
  @last_row-1,start_col+skip_cols*5-2 SAY "  "
  @last_row-1,start_col+skip_cols*6-2 SAY "  "
  @last_row-1,start_col+skip_cols*7-2 SAY "  "

  @last_row-3,start_col+skip_cols-2 SAY "  "
  @last_row-3,start_col+skip_cols*2-2 SAY "  "
  @last_row-3,start_col+skip_cols*3-2 SAY "  "
  @last_row-3,start_col+skip_cols*4-2 SAY "  "
  @last_row-3,start_col+skip_cols*5-2 SAY "  "
  @last_row-3,start_col+skip_cols*6-2 SAY "  "
  @last_row-3,start_col+skip_cols*7-2 SAY "  "
RETURN
*-- EOP: Clear_grid

*-- EOP: Calndar1.prg